/*
 *                          The Art Of
 *                      D E M O M A K I N G
 *
 *
 *                     by Alex J. Champandard
 *                          Base Sixteen
 *
 *                  Modified by Paulo Pinto (Moondevil)
 *                         for using with SDL
 *
 *
 *                http://www.flipcode.com/demomaking
 *
 *                This file is in the public domain.
 *                      Use at your own risk.
 *
 *
 *                 Compile with DJGPP:  make music
 *
 *
 */

#include <SDL.h>
#include <SDL_mixer.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <time.h>

using namespace std;

long long tickCount = 0;

static bool audio_open = false;
static Mix_Music *module = 0;

// Cleans all the resources. This is based in
// the CleanUp() function in playmus.c from
// SDL_mixer.
void CleanUp()
{
  if (Mix_PlayingMusic()) {
    Mix_FadeOutMusic(1500);
    SDL_Delay(1500);
  }
  if (module) {
    Mix_FreeMusic(module);
    module = 0;
  }
  if (audio_open) {
    Mix_CloseAudio();
    audio_open = false;
  }
  SDL_Quit();
}

/*
 * the main procedure
 */
int main(void)
{
    int audio_rate;
    Uint16 audio_format;
    int audio_channels;
    int audio_buffers;
    int audio_volume = MIX_MAX_VOLUME;
    int looping = 0;
    int interactive = 0;
    int i;

    // Initialize variables
    audio_rate = 22050;
    audio_format = AUDIO_S16;
    audio_channels = 2;
    audio_buffers = 4096;

    // Initialize the SDL library
    if (SDL_Init(SDL_INIT_AUDIO) < 0 ) {
      cerr << "Couldn't initialize SDL: " << SDL_GetError () << endl;
      exit( -1 );
    }
    atexit(CleanUp);

    // startup the SDL_mixer library
    // this code snippet was also borrowed from playmus.c and converted
    // to C++.
    if (Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers) < 0) {
      cerr << "Couldn't open audio: " << SDL_GetError() << endl;
      exit(-1);
    } else {
      Mix_QuerySpec(&audio_rate, &audio_format, &audio_channels);
      cout << "Opened audio at "
	   << audio_rate << " Hz "
	   << (audio_format&0xFF) << " bit "
	   << ((audio_channels > 1) ? "stereo " : "mono ")
	   << audio_buffers << " bytes audio buffer" << endl;
    }
    audio_open = true;

 // load Steve's cool module
    module = Mix_LoadMUS("lord.xm");

 // and start playing... we can forget about it now
    Mix_PlayMusic( module, 0 );

// run the main loop
    bool done=false;
    while (!done)
    {
      SDL_Event event;
      if (SDL_PollEvent( &event ))
      {
	switch (event.type) {
	case SDL_KEYDOWN: if (event.key.keysym.sym == SDLK_ESCAPE)
	  done = true;
	  break;
	  
	case SDL_QUIT: done = true;
	  break;
	}
      }
    }

// stop playing the module
    Mix_FreeMusic( module );
    module = 0;

    return 0;
}
