TEXTURE *bm, *cm;

// the look-up table
unsigned char *bump_lut;

// size of the spot light
#define LIGHTSIZE 2.4f

// contains the precalculated spotlight
unsigned char *bump_light;

/*
 * this needs a bump map and a colour map, and 2 light coordinates
 * it computes the output colour with the look up table
 */
void Bump( unsigned char *bm, unsigned char *cm, int lx1, int ly1, int lx2, int ly2, int zoom )
{
     int i, j, px, py, x, y, offs, c;
  // we skip the first line since there are no pixels above
  // to calculate the slope with
     offs = 320;
  // loop for all the other lines
     vga->Lock();
     Uint8 *page_draw = vga->GetSurfaceData();
     for (j=1; j<200; j++)
     {
      // likewise, skip first pixel since there are no pixels on the left
         page_draw[offs] = 0; offs++;
         for (i=1; i<320; i++)
         {
          // calculate coordinates of the pixel we need in light map
          // given the slope at this point, and the zoom coefficient
             px = (i*zoom>>8) + bm[offs-1] - bm[offs];
             py = (j*zoom>>8) + bm[offs-320] - bm[offs];
          // add the movement of the first light
             x = px + lx1;
             y = py + ly1;
          // check if the coordinates are inside the light buffer
             if ((y>=0) && (y<256) && (x>=0) && (x<256))
             // if so get the pixel
                c = bump_light[(y<<8)+x];
          // otherwise assume intensity 0
             else c = 0;
          // now do the same for the second light
             x = px + lx2;
             y = py + ly2;
          // this time we add the light's intensity to the first value
             if ((y>=0) && (y<256) && (x>=0) && (x<256))
                c += bump_light[(y<<8)+x];
          // make sure it's not too big
             if (c>255) c = 255;
          // look up the colour multiplied by the light coeficient
             page_draw[offs] = bump_lut[(cm[offs]<<8)+c];
             offs++;
         }
     }
     vga->Unlock();
}

/*
 * given a palette compute the best lookup table containing 256 shades
 * of each original colour
 */
void Compute_LUT3( unsigned char *pal )
{
   unsigned char r, g, b, l;
// for each colour
   for (int j=0; j<256; j++)
   {
       r = pal[j*3];
       g = pal[j*3+1];
       b = pal[j*3+2];
    // for all 256 intensities
       for (int i=0; i<256; i++)
       {
        // get the index of the best colour
           bump_lut[(j<<8)+i] = Closest_Colour( pal, (r*i)/255, (g*i)/255, (b*i)/255 );
       }
   }
}

/*
 * generate a "spot light" pattern
 */
void Compute_Light()
{
    for (int j=0; j<256; j++)
        for (int i=0; i<256; i++)
        {
         // get the distance from the centre
            float dist = (128-i)*(128-i) + (128-j)*(128-j);
            if (fabs(dist)>1) dist = sqrt( dist );
         // then fade if according to the distance, and a random coefficient
            int c = (int)(LIGHTSIZE*dist)+(rand()&7)-3;
         // clip it
            if (c<0) c = 0;
            if (c>255) c = 255;
         // and store it
            bump_light[(j<<8)+i] = 255-c;
        }
}

void Bump_Init()
{
// load the bump map and the colour map
   bm = new TEXTURE( "bumpmap.png" );
   cm = new TEXTURE( "colmap.png" );
// compute the lookup table
   bump_lut = new (unsigned char)[65536];
   Compute_LUT3( cm->palette );
// contains the image of the spotlight
   bump_light = new (unsigned char)[65536];
// generate the light pattern
   Compute_Light();
}

void Bump_Run()
{
// light movement
    int x1, y1, x2, y2, z;
    bQuit = false;
// run the main loop
    while (!kbhit() && !bQuit)
    {
      for (int i=0; i<256; i++)
	{
           vga->SetColour( i,
           (unsigned char)((cm->palette[i*3]>>2)*(1-flash_coef)+63*flash_coef),
           (unsigned char)((cm->palette[i*3+1]>>2)*(1-flash_coef)+63*flash_coef),
           (unsigned char)((cm->palette[i*3+2]>>2)*(1-flash_coef)+63*flash_coef) );
       }
       long long currentTime = timer->getCount()>>14;
    // move the light.... more sines :)
       x1 = (int)(128.0f * cos( (double)currentTime/64 ))-20;
       y1 = (int)(128.0f * sin( (double)-currentTime/45 ))+20;
       x2 = (int)(128.0f * cos( (double)-currentTime/51 ))-20;
       y2 = (int)(128.0f * sin( (double)currentTime/71 ))+20;
       z = 192+(int)(127.0f * sin( (double)currentTime/112 ));
    // draw the bumped image
       Bump( bm->location, cm->location, x1, y1, x2, y2, z );
    // dump to screen
       vga->Update();
       frameCount++;
    }
}

void Bump_Done()
{
// free memory
    delete bm;
    delete cm;
    delete bump_light;
    delete bump_lut;
}
